/* This file is part of Mailfromd.
   Copyright (C) 2009 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "c-ctype.h"
#include "mailfromd.h"
#include "prog.h"

int
check_pragma_args(const struct pragma *pdef, int argc)
{
	if (pdef->minargs > 0 && argc < pdef->minargs) {
		parse_error(_("too few arguments in pragma"));
		return 1;
	}
	if (pdef->maxargs > 0 && argc > pdef->maxargs) {
		parse_error(_("too many arguments in pragma"));
		return 1;
	}
	return 0;
}

void
parse_pragma(const char *text)
{
	int rc;
	int argc;
	char **argv;
	const struct pragma *pdef;
	
	while (*text != '#')
		text++;
	++text;
	while (*text && c_isspace(*text)) text++;
	text += 6; /* "pragma" */

	rc = mu_argcv_get(text, "", NULL, &argc, &argv);
	if (rc) {
		parse_error(_("error parsing pragma: %s"), mu_strerror (rc));
		return;
	}

	if (argc == 0) {
		parse_error(_("Empty pragma"));
		return;
	}

	pdef = lookup_pragma(argv[0]);
	if (!pdef) 
		parse_error(_("Unknown pragma"));
	else if (check_pragma_args(pdef, argc) == 0)
		pdef->handler(argc, argv, text);
	mu_argcv_free(argc, argv);
}
